@@ -4,10 +4,10 @@
 
 package mozilla.components.browser.engine.gecko
 
-import android.net.Uri
 import android.os.Build
 import android.view.WindowManager
 import androidx.annotation.VisibleForTesting
+import androidx.core.net.toUri
 import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.Dispatchers
 import kotlinx.coroutines.Job
@@ -36,10 +36,6 @@ import mozilla.components.concept.engine.manifest.WebAppManifest
 import mozilla.components.concept.engine.manifest.WebAppManifestParser
 import mozilla.components.concept.engine.request.RequestInterceptor
 import mozilla.components.concept.engine.request.RequestInterceptor.InterceptionResponse
-import mozilla.components.concept.engine.shopping.Highlight
-import mozilla.components.concept.engine.shopping.ProductAnalysis
-import mozilla.components.concept.engine.shopping.ProductAnalysisStatus
-import mozilla.components.concept.engine.shopping.ProductRecommendation
 import mozilla.components.concept.engine.translate.TranslationError
 import mozilla.components.concept.engine.translate.TranslationOperation
 import mozilla.components.concept.engine.translate.TranslationOptions
@@ -57,7 +53,6 @@ import mozilla.components.support.base.facts.Action
 import mozilla.components.support.base.facts.Fact
 import mozilla.components.support.base.facts.collect
 import mozilla.components.support.base.log.logger.Logger
-import mozilla.components.support.ktx.kotlin.decode
 import mozilla.components.support.ktx.kotlin.isEmail
 import mozilla.components.support.ktx.kotlin.isExtensionUrl
 import mozilla.components.support.ktx.kotlin.isGeoLocation
@@ -75,7 +70,6 @@ import org.mozilla.geckoview.GeckoRuntime
 import org.mozilla.geckoview.GeckoSession
 import org.mozilla.geckoview.GeckoSession.NavigationDelegate
 import org.mozilla.geckoview.GeckoSession.PermissionDelegate.ContentPermission
-import org.mozilla.geckoview.GeckoSession.Recommendation
 import org.mozilla.geckoview.GeckoSessionSettings
 import org.mozilla.geckoview.WebRequestError
 import org.mozilla.geckoview.WebResponse
@@ -179,10 +173,12 @@ class GeckoEngineSession(
         parent: EngineSession?,
         flags: LoadUrlFlags,
         additionalHeaders: Map<String, String>?,
+        originalInput: String?,
+        textDirectiveUserActivation: Boolean,
     ) {
         notifyObservers { onLoadUrl() }
 
-        val scheme = Uri.parse(url).normalizeScheme().scheme
+        val scheme = url.toUri().normalizeScheme().scheme
         if (BLOCKED_SCHEMES.contains(scheme) && !shouldLoadJSSchemes(scheme, flags)) {
             logger.error("URL scheme not allowed. Aborting load.")
             return
@@ -195,6 +191,8 @@ class GeckoEngineSession(
         val loader = GeckoSession.Loader()
             .uri(url)
             .flags(flags.getGeckoFlags())
+            .originalInput(originalInput)
+            .textDirectiveUserActivation(textDirectiveUserActivation)
 
         if (additionalHeaders != null) {
             val headerFilter = if (flags.contains(ALLOW_ADDITIONAL_HEADERS)) {
@@ -256,7 +254,6 @@ class GeckoEngineSession(
                 // See https://github.com/mozilla-mobile/android-components/issues/12276
                 val fileName = DownloadUtils.guessFileName(
                     disposition,
-                    destinationDirectory = null,
                     url = url,
                     mimeType = contentType,
                 )
@@ -463,6 +460,9 @@ class GeckoEngineSession(
 
         val newViewportMode = if (enable) {
             overrideUrl = currentUrl?.let { checkForMobileSite(it) }
+            if (overrideUrl == null && pageLoadingUrl != currentUrl) {
+                overrideUrl = pageLoadingUrl
+            }
             GeckoSessionSettings.VIEWPORT_MODE_DESKTOP
         } else {
             GeckoSessionSettings.VIEWPORT_MODE_MOBILE
@@ -523,7 +523,7 @@ class GeckoEngineSession(
         val mPrefix = "m."
         val mobilePrefix = "mobile."
 
-        val uri = Uri.parse(url)
+        val uri = url.toUri()
         val authority = uri.authority?.lowercase(Locale.ROOT) ?: return null
 
         val foundPrefix = when {
@@ -533,7 +533,7 @@ class GeckoEngineSession(
         }
 
         foundPrefix?.let {
-            val mobileUri = Uri.parse(url).buildUpon().authority(authority.substring(it.length))
+            val mobileUri = url.toUri().buildUpon().authority(authority.substring(it.length))
             overrideUrl = mobileUri.toString()
         }
 
@@ -608,6 +608,13 @@ class GeckoEngineSession(
         }
     }
 
+    /**
+     * See [EngineSession.onPipModeChanged].
+     */
+    override fun onPipModeChanged(enabled: Boolean) {
+        geckoSession.compositorController.onPipModeChanged(enabled)
+    }
+
     /**
      * See [EngineSession.checkForFormData].
      */
@@ -662,44 +669,23 @@ class GeckoEngineSession(
     }
 
     /**
-     * See [EngineSession.requestProductRecommendations]
+     * See [EngineSession.getWebCompatInfo].
      */
-    override fun requestProductRecommendations(
-        url: String,
-        onResult: (List<ProductRecommendation>) -> Unit,
+    override fun getWebCompatInfo(
+        onResult: (JSONObject) -> Unit,
         onException: (Throwable) -> Unit,
     ) {
-        geckoSession.requestRecommendations(url).then(
-            { response: List<Recommendation>? ->
-                if (response == null) {
-                    logger.error("Invalid value: unable to get analysis result from Gecko Engine.")
-                    onException(
-                        java.lang.IllegalStateException(
-                            "Invalid value: unable to get analysis result from Gecko Engine.",
-                        ),
-                    )
-                    return@then GeckoResult()
+        geckoSession.webCompatInfo.then(
+            { result ->
+                if (result == null) {
+                    logger.error("No result from GeckoView getWebCompatInfo.")
+                    return@then GeckoResult<JSONObject>()
                 }
-
-                val productRecommendations = response.map { it: Recommendation ->
-                    ProductRecommendation(
-                        url = it.url,
-                        analysisUrl = it.analysisUrl,
-                        adjustedRating = it.adjustedRating,
-                        sponsored = it.sponsored,
-                        imageUrl = it.imageUrl,
-                        aid = it.aid,
-                        name = it.name,
-                        grade = it.grade,
-                        price = it.price,
-                        currency = it.currency,
-                    )
-                }
-                onResult(productRecommendations)
-                GeckoResult<ProductRecommendation>()
+                onResult(result)
+                GeckoResult()
             },
             { throwable ->
-                logger.error("Requesting product analysis failed.", throwable)
+                logger.error("Getting web compat info failed.", throwable)
                 onException(throwable)
                 GeckoResult()
             },
@@ -707,243 +693,20 @@ class GeckoEngineSession(
     }
 
     /**
-     * See [EngineSession.requestProductAnalysis]
+     * See [EngineSession.sendMoreWebCompatInfo].
      */
-    @Suppress("ComplexCondition")
-    override fun requestProductAnalysis(
-        url: String,
-        onResult: (ProductAnalysis) -> Unit,
+    override fun sendMoreWebCompatInfo(
+        info: JSONObject,
+        onResult: () -> Unit,
         onException: (Throwable) -> Unit,
     ) {
-        geckoSession.requestAnalysis(url).then(
-            { response ->
-                if (response == null) {
-                    logger.error(
-                        "Invalid value: unable to get analysis result from Gecko Engine.",
-                    )
-                    onException(
-                        java.lang.IllegalStateException(
-                            "Invalid value: unable to get analysis result from Gecko Engine.",
-                        ),
-                    )
-                    return@then GeckoResult()
-                }
-
-                val highlights = if (
-                    response.highlights?.quality == null &&
-                    response.highlights?.price == null &&
-                    response.highlights?.shipping == null &&
-                    response.highlights?.appearance == null &&
-                    response.highlights?.competitiveness == null
-                ) {
-                    null
-                } else {
-                    Highlight(
-                        response.highlights?.quality?.toList(),
-                        response.highlights?.price?.toList(),
-                        response.highlights?.shipping?.toList(),
-                        response.highlights?.appearance?.toList(),
-                        response.highlights?.competitiveness?.toList(),
-                    )
-                }
-
-                val analysisResult = ProductAnalysis(
-                    productId = response.productId,
-                    analysisURL = response.analysisURL,
-                    grade = response.grade,
-                    adjustedRating = response.adjustedRating,
-                    needsAnalysis = response.needsAnalysis,
-                    pageNotSupported = response.pageNotSupported,
-                    notEnoughReviews = response.notEnoughReviews,
-                    lastAnalysisTime = response.lastAnalysisTime,
-                    deletedProductReported = response.deletedProductReported,
-                    deletedProduct = response.deletedProduct,
-                    highlights = highlights,
-                )
-
-                onResult(analysisResult)
-                GeckoResult<ProductAnalysis>()
+        geckoSession.sendMoreWebCompatInfo(info).then(
+            {
+                onResult()
+                GeckoResult<Void>()
             },
             { throwable ->
-                logger.error("Requesting product analysis failed.", throwable)
-                onException(throwable)
-                GeckoResult()
-            },
-        )
-    }
-
-    /**
-     * See [EngineSession.reanalyzeProduct]
-     */
-    override fun reanalyzeProduct(
-        url: String,
-        onResult: (String) -> Unit,
-        onException: (Throwable) -> Unit,
-    ) {
-        geckoSession.requestCreateAnalysis(url).then(
-            { response ->
-                val errorMessage = "Invalid value: unable to reanalyze product from Gecko Engine."
-                if (response == null) {
-                    logger.error(errorMessage)
-                    onException(
-                        java.lang.IllegalStateException(errorMessage),
-                    )
-                    return@then GeckoResult()
-                }
-                onResult(response)
-                GeckoResult<String>()
-            },
-            { throwable ->
-                logger.error("Request to reanalyze product failed.", throwable)
-                onException(throwable)
-                GeckoResult()
-            },
-        )
-    }
-
-    /**
-     * See [EngineSession.requestAnalysisStatus]
-     */
-    override fun requestAnalysisStatus(
-        url: String,
-        onResult: (ProductAnalysisStatus) -> Unit,
-        onException: (Throwable) -> Unit,
-    ) {
-        geckoSession.requestAnalysisStatus(url).then(
-            { response ->
-                val errorMessage = "Invalid value: unable to request analysis status from Gecko Engine."
-                if (response == null) {
-                    logger.error(errorMessage)
-                    onException(
-                        java.lang.IllegalStateException(errorMessage),
-                    )
-                    return@then GeckoResult()
-                }
-                val analysisStatusResult = ProductAnalysisStatus(
-                    status = response.status,
-                    progress = response.progress,
-                )
-                onResult(analysisStatusResult)
-                GeckoResult<ProductAnalysisStatus>()
-            },
-            { throwable ->
-                logger.error("Request for product analysis status failed.", throwable)
-                onException(throwable)
-                GeckoResult()
-            },
-        )
-    }
-
-    /**
-     * See [EngineSession.sendClickAttributionEvent]
-     */
-    override fun sendClickAttributionEvent(
-        aid: String,
-        onResult: (Boolean) -> Unit,
-        onException: (Throwable) -> Unit,
-    ) {
-        geckoSession.sendClickAttributionEvent(aid).then(
-            { response ->
-                val errorMessage = "Invalid value: unable to send click attribution event through Gecko Engine."
-                if (response == null) {
-                    logger.error(errorMessage)
-                    onException(
-                        java.lang.IllegalStateException(errorMessage),
-                    )
-                    return@then GeckoResult()
-                }
-                onResult(response)
-                GeckoResult<Boolean>()
-            },
-            { throwable ->
-                logger.error("Sending click attribution event failed.", throwable)
-                onException(throwable)
-                GeckoResult()
-            },
-        )
-    }
-
-    /**
-     * See [EngineSession.sendImpressionAttributionEvent]
-     */
-    override fun sendImpressionAttributionEvent(
-        aid: String,
-        onResult: (Boolean) -> Unit,
-        onException: (Throwable) -> Unit,
-    ) {
-        geckoSession.sendImpressionAttributionEvent(aid).then(
-            { response ->
-                val errorMessage = "Invalid value: unable to send impression attribution event through Gecko Engine."
-                if (response == null) {
-                    logger.error(errorMessage)
-                    onException(
-                        java.lang.IllegalStateException(errorMessage),
-                    )
-                    return@then GeckoResult()
-                }
-                onResult(response)
-                GeckoResult<Boolean>()
-            },
-            { throwable ->
-                logger.error("Sending impression attribution event failed.", throwable)
-                onException(throwable)
-                GeckoResult()
-            },
-        )
-    }
-
-    /**
-     * See [EngineSession.sendPlacementAttributionEvent]
-     */
-    override fun sendPlacementAttributionEvent(
-        aid: String,
-        onResult: (Boolean) -> Unit,
-        onException: (Throwable) -> Unit,
-    ) {
-        geckoSession.sendPlacementAttributionEvent(aid).then(
-            { response ->
-                val errorMessage = "Invalid value: unable to send placement attribution event through Gecko Engine."
-                if (response == null) {
-                    logger.error(errorMessage)
-                    onException(
-                        java.lang.IllegalStateException(errorMessage),
-                    )
-                    return@then GeckoResult()
-                }
-                onResult(response)
-                GeckoResult<Boolean>()
-            },
-            { throwable ->
-                logger.error("Sending placement attribution event failed.", throwable)
-                onException(throwable)
-                GeckoResult()
-            },
-        )
-    }
-
-    /**
-     * See [EngineSession.reportBackInStock]
-     */
-    override fun reportBackInStock(
-        url: String,
-        onResult: (String) -> Unit,
-        onException: (Throwable) -> Unit,
-    ) {
-        geckoSession.reportBackInStock(url).then(
-            { response ->
-                val errorMessage = "Invalid value: unable to report back in stock from Gecko Engine."
-                if (response == null) {
-                    logger.error(errorMessage)
-                    onException(
-                        java.lang.IllegalStateException(errorMessage),
-                    )
-                    return@then GeckoResult()
-                }
-                onResult(response)
-                GeckoResult<String>()
-            },
-            { throwable ->
-                logger.error("Request for reporting back in stock failed.", throwable)
+                logger.error("Sending more web compat info failed.", throwable)
                 onException(throwable)
                 GeckoResult()
             },
@@ -980,8 +743,7 @@ class GeckoEngineSession(
                 onTranslateComplete(TranslationOperation.TRANSLATE)
             }
             GeckoResult<Void>()
-        }, {
-                throwable ->
+        }, { throwable ->
             logger.error("Request for translation failed: ", throwable)
             notifyObservers {
                 onTranslateException(
@@ -1012,8 +774,7 @@ class GeckoEngineSession(
                 onTranslateComplete(TranslationOperation.RESTORE)
             }
             GeckoResult<Void>()
-        }, {
-                throwable ->
+        }, { throwable ->
             logger.error("Request for translation failed: ", throwable)
             notifyObservers {
                 onTranslateException(TranslationOperation.RESTORE, throwable.intoTranslationError())
@@ -1034,8 +795,7 @@ class GeckoEngineSession(
             return
         }
 
-        geckoSession.sessionTranslation!!.neverTranslateSiteSetting.then({
-                response ->
+        geckoSession.sessionTranslation!!.neverTranslateSiteSetting.then({ response ->
             if (response == null) {
                 logger.error("Did not receive a site setting response.")
                 onException(
@@ -1045,8 +805,7 @@ class GeckoEngineSession(
             }
             onResult(response)
             GeckoResult<Boolean>()
-        }, {
-                throwable ->
+        }, { throwable ->
             logger.error("Request for site translation preference failed: ", throwable)
             onException(throwable.intoTranslationError())
             GeckoResult()
@@ -1069,8 +828,7 @@ class GeckoEngineSession(
         geckoSession.sessionTranslation!!.setNeverTranslateSiteSetting(setting).then({
             onResult()
             GeckoResult<Boolean>()
-        }, {
-                throwable ->
+        }, { throwable ->
             logger.error("Request for setting site translation preference failed: ", throwable)
             onException(throwable.intoTranslationError())
             GeckoResult()
@@ -1120,10 +878,13 @@ class GeckoEngineSession(
                 return
             }
 
-            appRedirectUrl?.let {
-                if (url == appRedirectUrl) {
-                    goBack(false)
-                    return
+            // if it is an initial load then we can't go back. We should update the URL.
+            if (!initialLoad) {
+                appRedirectUrl?.let {
+                    if (url == appRedirectUrl) {
+                        goBack(false)
+                        return
+                    }
                 }
             }
 
@@ -1140,6 +901,8 @@ class GeckoEngineSession(
             }
             // Reset the status of current page being product or not when user navigates away.
             notifyObservers { onProductUrlChange(false) }
+            // Reset the status of the translation state for the page
+            notifyObservers { onTranslatePageChange() }
             notifyObservers { onLocationChange(url, hasUserGesture) }
         }
 
@@ -1237,8 +1000,10 @@ class GeckoEngineSession(
 
             val interceptor = settings.requestInterceptor
             val interceptionResponse = if (
-                interceptor != null && (!request.isDirectNavigation || interceptor.interceptsAppInitiatedRequests())
+                interceptor == null || (request.isDirectNavigation && !interceptor.interceptsAppInitiatedRequests())
             ) {
+                null
+            } else {
                 val engineSession = this@GeckoEngineSession
                 val isSameDomain =
                     engineSession.currentUrl?.tryGetHostFromUrl() == request.uri.tryGetHostFromUrl()
@@ -1251,27 +1016,33 @@ class GeckoEngineSession(
                     request.isRedirect,
                     request.isDirectNavigation,
                     isSubframeRequest,
-                )?.apply {
+                )?.takeUnless {
+                    it is InterceptionResponse.AppIntent && request.isDirectNavigation
+                }?.apply {
                     when (this) {
+                        is InterceptionResponse.AppIntent -> {
+                            appRedirectUrl = lastLoadRequestUri
+                            notifyObservers {
+                                onLaunchIntentRequest(
+                                    url = url,
+                                    appIntent = appIntent,
+                                    fallbackUrl = fallbackUrl,
+                                    appName = appName,
+                                )
+                            }
+                        }
+
                         is InterceptionResponse.Content -> loadData(data, mimeType, encoding)
                         is InterceptionResponse.Url -> loadUrl(
                             url = url,
                             flags = flags,
                             additionalHeaders = additionalHeaders,
                         )
-                        is InterceptionResponse.AppIntent -> {
-                            appRedirectUrl = lastLoadRequestUri
-                            notifyObservers {
-                                onLaunchIntentRequest(url = url, appIntent = appIntent)
-                            }
-                        }
                         else -> {
                             // no-op
                         }
                     }
                 }
-            } else {
-                null
             }
 
             if (interceptionResponse !is InterceptionResponse.AppIntent) {
@@ -1511,7 +1282,6 @@ class GeckoEngineSession(
                 val url = uri
                 val fileName = DownloadUtils.guessFileName(
                     contentDisposition,
-                    destinationDirectory = null,
                     url = url,
                     mimeType = contentType,
                 )
@@ -1521,7 +1291,7 @@ class GeckoEngineSession(
                         url = url,
                         contentLength = contentLength,
                         contentType = DownloadUtils.sanitizeMimeType(contentType),
-                        fileName = fileName.sanitizeFileName().decode(),
+                        fileName = fileName.sanitizeFileName(),
                         response = response,
                         isPrivate = privateMode,
                         openInApp = webResponse.requestExternalApp,
@@ -1699,7 +1469,7 @@ class GeckoEngineSession(
             val geckoResult = GeckoResult<Int>()
             val uri = geckoContentPermission.uri
             val type = geckoContentPermission.permission
-            val request = GeckoPermissionRequest.Content(uri, type, geckoContentPermission, geckoResult)
+            val request = GeckoPermissionRequest.Content(uri, type, geckoContentPermission, mutableListOf(geckoResult))
             notifyObservers { onContentPermissionRequest(request) }
             return geckoResult
         }
@@ -1815,7 +1585,7 @@ class GeckoEngineSession(
         internal const val ABOUT_BLANK = "about:blank"
         internal const val JS_SCHEME = "javascript"
         internal val BLOCKED_SCHEMES =
-            listOf("file", "resource", JS_SCHEME) // See 1684761 and 1684947
+            listOf("file", "resource", "fido", JS_SCHEME) // See 1684761 and 1684947
 
         /**
          * Provides an ErrorType corresponding to the error code provided.
