@@ -28,7 +28,7 @@ import mozilla.components.browser.state.state.UndoHistoryState
 import mozilla.components.browser.state.state.WebExtensionState
 import mozilla.components.browser.state.state.content.DownloadState
 import mozilla.components.browser.state.state.content.FindResultState
-import mozilla.components.browser.state.state.content.ShareInternetResourceState
+import mozilla.components.browser.state.state.content.ShareResourceState
 import mozilla.components.browser.state.state.extension.WebExtensionPromptRequest
 import mozilla.components.browser.state.state.recover.RecoverableTab
 import mozilla.components.browser.state.state.recover.TabState
@@ -378,6 +378,11 @@ sealed class UndoAction : BrowserAction() {
      * Restores the tabs in [UndoHistoryState].
      */
     object RestoreRecoverableTabs : UndoAction()
+
+    /**
+     * Updates the [EngineState] for the given tab id in [UndoHistoryState].
+     */
+    data class UpdateEngineStateForRecoverableTab(val id: String, val engineState: EngineSessionState) : UndoAction()
 }
 
 /**
@@ -810,7 +815,7 @@ sealed class ContentAction : BrowserAction() {
     /**
      * Updates the [ContentState] of the given [sessionId] to indicate whether or not desktop mode is enabled.
      */
-    data class UpdateDesktopModeAction(val sessionId: String, val enabled: Boolean) : ContentAction()
+    data class UpdateTabDesktopMode(val sessionId: String, val enabled: Boolean) : ContentAction()
 
     /**
      * Updates the [AppIntentState] of the [ContentState] with the given [sessionId].
@@ -855,6 +860,20 @@ sealed class ContentAction : BrowserAction() {
         val tabId: String,
         val isProductUrl: Boolean,
     ) : ContentAction()
+
+    /**
+     * Inform that the tab with [tabId] started rendering a pdf.
+     */
+    data class EnteredPdfViewer(
+        val tabId: String,
+    ) : ContentAction()
+
+    /**
+     * Inform that the tab with [tabId] stopped rendering a pdf.
+     */
+    data class ExitedPdfViewer(
+        val tabId: String,
+    ) : ContentAction()
 }
 
 /**
@@ -1040,6 +1059,19 @@ sealed class TranslationsAction : BrowserAction() {
         val pageSettings: TranslationPageSettings?,
     ) : TranslationsAction(), ActionWithTab
 
+    /**
+     * Indicates the translation processing state on the given [tabId].
+     *
+     * A translation is processing when the engine is actively working on performing the translation.
+     *
+     * @property tabId The ID of the tab the [EngineSession] should be linked to.
+     * @property isProcessing Whether the translation is processing or not.
+     */
+    data class SetTranslateProcessingAction(
+        override val tabId: String,
+        val isProcessing: Boolean,
+    ) : TranslationsAction(), ActionWithTab
+
     /**
      * Updates the specified page setting operation on the translation engine and ensures the final
      * state on the given [tabId]'s store remains in-sync.
@@ -1318,6 +1350,7 @@ sealed class EngineAction : BrowserAction() {
         val flags: EngineSession.LoadUrlFlags = EngineSession.LoadUrlFlags.none(),
         val additionalHeaders: Map<String, String>? = null,
         val includeParent: Boolean = false,
+        val textDirectiveUserActivation: Boolean = false,
     ) : EngineAction(), ActionWithTab
 
     /**
@@ -1681,19 +1714,27 @@ sealed class DownloadAction : BrowserAction() {
      * Restores the given [download] from the storage.
      */
     data class RestoreDownloadStateAction(val download: DownloadState) : DownloadAction()
+
+    /**
+     * [BrowserAction] to remove downloads from the storage that no longer exist on disk.
+     *
+     * This action is used to clean up the download storage by removing entries for files
+     * that have been deleted or moved from their original download location.
+     */
+    data object RemoveDeletedDownloads : DownloadAction()
 }
 
 /**
- * [BrowserAction] implementations related to updating the session state of internet resources to be shared.
+ * [BrowserAction] implementations related to updating the session state of internet or local resources to be shared.
  */
-sealed class ShareInternetResourceAction : BrowserAction() {
+sealed class ShareResourceAction : BrowserAction() {
     /**
-     * Starts the sharing process of an Internet resource.
+     * Starts the sharing process of an Internet or local resource.
      */
     data class AddShareAction(
         val tabId: String,
-        val internetResource: ShareInternetResourceState,
-    ) : ShareInternetResourceAction()
+        val resource: ShareResourceState,
+    ) : ShareResourceAction()
 
     /**
      * Previous share request is considered completed.
@@ -1702,7 +1743,7 @@ sealed class ShareInternetResourceAction : BrowserAction() {
      */
     data class ConsumeShareAction(
         val tabId: String,
-    ) : ShareInternetResourceAction()
+    ) : ShareResourceAction()
 }
 
 /**
@@ -1714,7 +1755,7 @@ sealed class CopyInternetResourceAction : BrowserAction() {
      */
     data class AddCopyAction(
         val tabId: String,
-        val internetResource: ShareInternetResourceState,
+        val internetResource: ShareResourceState.InternetResource,
     ) : CopyInternetResourceAction()
 
     /**
@@ -1783,6 +1824,11 @@ sealed class SearchAction : BrowserAction() {
      */
     data class SetRegionAction(val regionState: RegionState, val distribution: String? = null) : SearchAction()
 
+    /**
+     * Application Search Engines have finished loading from disk.
+     */
+    data class ApplicationSearchEnginesLoaded(val applicationSearchEngines: List<SearchEngine>) : SearchAction()
+
     /**
      * Sets the list of search engines and default search engine IDs.
      */
@@ -1856,6 +1902,11 @@ sealed class SearchAction : BrowserAction() {
     object RestoreHiddenSearchEnginesAction : SearchAction()
 }
 
+/**
+ * [BrowserAction] implements setting and updating the distribution
+ */
+data class UpdateDistribution(val distributionId: String?) : BrowserAction()
+
 /**
  * [BrowserAction] implementations for updating state needed for debugging. These actions should
  * be carefully considered before being used.
@@ -1890,3 +1941,18 @@ sealed class AppLifecycleAction : BrowserAction() {
      */
     object PauseAction : AppLifecycleAction()
 }
+
+/**
+ * [BrowserAction] implementations related to updating the application's default desktop mode setting.
+ */
+sealed class DefaultDesktopModeAction : BrowserAction() {
+    /**
+     * Toggles the global default for desktop browsing mode.
+     */
+    data object ToggleDesktopMode : DefaultDesktopModeAction()
+
+    /**
+     * Updates the global default for desktop browsing mode.
+     */
+    data class DesktopModeUpdated(val newValue: Boolean) : DefaultDesktopModeAction()
+}
