@@ -4,51 +4,63 @@
 
 package org.mozilla.geckoview.test
 
-import android.os.Build
 import android.os.SystemClock
-import androidx.test.platform.app.InstrumentationRegistry
 import androidx.test.filters.MediumTest
-import androidx.test.filters.SdkSuppress
-import androidx.test.ext.junit.runners.AndroidJUnit4
+import androidx.test.platform.app.InstrumentationRegistry
 import kotlinx.coroutines.Dispatchers
 import kotlinx.coroutines.runBlocking
 import org.hamcrest.MatcherAssert.assertThat
-import org.hamcrest.Matchers.*
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.greaterThan
+import org.hamcrest.Matchers.isEmptyOrNullString
+import org.hamcrest.Matchers.not
+import org.hamcrest.Matchers.notNullValue
 import org.json.JSONObject
 import org.junit.After
+import org.junit.Assert.assertThrows
 import org.junit.Before
-import org.junit.Rule
 import org.junit.Test
-import org.junit.rules.ExpectedException
 import org.junit.runner.RunWith
+import org.junit.runners.Parameterized
+import org.mozilla.gecko.util.ThreadUtils
+import org.mozilla.geckoview.GeckoResult
 import org.mozilla.geckoview.GeckoWebExecutor
+import org.mozilla.geckoview.StorageController
 import org.mozilla.geckoview.WebRequest
 import org.mozilla.geckoview.WebRequestError
 import org.mozilla.geckoview.WebResponse
 import org.mozilla.geckoview.test.util.RuntimeCreator
 import org.mozilla.geckoview.test.util.TestServer
 import java.io.IOException
-import java.lang.IllegalStateException
 import java.math.BigInteger
 import java.net.UnknownHostException
 import java.nio.ByteBuffer
 import java.nio.charset.Charset
 import java.security.MessageDigest
-import java.util.*
+import java.util.Random
 
 @MediumTest
-@RunWith(AndroidJUnit4::class)
+@RunWith(Parameterized::class)
 class WebExecutorTest {
     companion object {
         const val TEST_PORT: Int = 4242
-        const val TEST_ENDPOINT: String = "http://localhost:${TEST_PORT}"
+        const val TEST_ENDPOINT: String = "http://localhost:$TEST_PORT"
+
+        @get:Parameterized.Parameters(name = "{0}")
+        @JvmStatic
+        val parameters: List<Array<out Any>> = listOf(
+            arrayOf("#conservative"),
+            arrayOf("#normal"),
+        )
     }
 
+    @field:Parameterized.Parameter(0)
+    @JvmField
+    var id: String = ""
+
     lateinit var executor: GeckoWebExecutor
     lateinit var server: TestServer
 
-    @get:Rule val thrown = ExpectedException.none()
-
     @Before
     fun setup() {
         // Using @UiThreadTest here does not seem to block
@@ -100,22 +112,34 @@ class WebExecutorTest {
         return builder.toString()
     }
 
+    fun webRequestBuilder(uri: String): WebRequest.Builder {
+        val beConservative = when (id) {
+            "#conservative" -> true
+            else -> false
+        }
+        return WebRequest.Builder(uri).beConservative(beConservative)
+    }
+
+    fun webRequest(uri: String): WebRequest {
+        return webRequestBuilder(uri).build()
+    }
+
     @Test
     fun smoke() {
         val uri = "$TEST_ENDPOINT/anything"
         val bodyString = randomString(8192)
         val referrer = "http://foo/bar"
 
-        val request = WebRequest.Builder(uri)
-                .method("POST")
-                .header("Header1", "Clobbered")
-                .header("Header1", "Value")
-                .addHeader("Header2", "Value1")
-                .addHeader("Header2", "Value2")
-                .referrer(referrer)
-                .header("Content-Type", "text/plain")
-                .body(bodyString)
-                .build()
+        val request = webRequestBuilder(uri)
+            .method("POST")
+            .header("Header1", "Clobbered")
+            .header("Header1", "Value")
+            .addHeader("Header2", "Value1")
+            .addHeader("Header2", "Value2")
+            .referrer(referrer)
+            .header("Content-Type", "text/plain")
+            .body(bodyString)
+            .build()
 
         val response = fetch(request)
 
@@ -131,34 +155,34 @@ class WebExecutorTest {
         assertThat("Headers should match", body.getJSONObject("headers").getString("Header2"), equalTo("Value1, Value2"))
         assertThat("Headers should match", body.getJSONObject("headers").getString("Content-Type"), equalTo("text/plain"))
         assertThat("Referrer should match", body.getJSONObject("headers").getString("Referer"), equalTo("http://foo/"))
-        assertThat("Data should match", body.getString("data"), equalTo(bodyString));
+        assertThat("Data should match", body.getString("data"), equalTo(bodyString))
     }
 
     @Test
     fun testFetchAsset() {
-        val response = fetch(WebRequest("$TEST_ENDPOINT/assets/www/hello.html"))
+        val response = fetch(webRequest("$TEST_ENDPOINT/assets/www/hello.html"))
         assertThat("Status should match", response.statusCode, equalTo(200))
         assertThat("Body should have bytes", response.getBodyBytes().remaining(), greaterThan(0))
     }
 
     @Test
     fun testStatus() {
-        val response = fetch(WebRequest("$TEST_ENDPOINT/status/500"))
+        val response = fetch(webRequest("$TEST_ENDPOINT/status/500"))
         assertThat("Status code should match", response.statusCode, equalTo(500))
     }
 
     @Test
     fun testRedirect() {
-        val response = fetch(WebRequest("$TEST_ENDPOINT/redirect-to?url=/status/200"))
+        val response = fetch(webRequest("$TEST_ENDPOINT/redirect-to?url=/status/200"))
 
-        assertThat("URI should match", response.uri, equalTo(TEST_ENDPOINT +"/status/200"))
+        assertThat("URI should match", response.uri, equalTo(TEST_ENDPOINT + "/status/200"))
         assertThat("Redirected should match", response.redirected, equalTo(true))
         assertThat("Status code should match", response.statusCode, equalTo(200))
     }
 
     @Test
     fun testDisallowRedirect() {
-        val response = fetch(WebRequest("$TEST_ENDPOINT/redirect-to?url=/status/200"), GeckoWebExecutor.FETCH_FLAGS_NO_REDIRECTS)
+        val response = fetch(webRequest("$TEST_ENDPOINT/redirect-to?url=/status/200"), GeckoWebExecutor.FETCH_FLAGS_NO_REDIRECTS)
 
         assertThat("URI should match", response.uri, equalTo("$TEST_ENDPOINT/redirect-to?url=/status/200"))
         assertThat("Redirected should match", response.redirected, equalTo(false))
@@ -167,15 +191,17 @@ class WebExecutorTest {
 
     @Test
     fun testRedirectLoop() {
-        thrown.expect(equalTo(WebRequestError(WebRequestError.ERROR_REDIRECT_LOOP, WebRequestError.ERROR_CATEGORY_NETWORK)))
-        fetch(WebRequest("$TEST_ENDPOINT/redirect/100"))
+        val thrown = assertThrows(WebRequestError::class.java) {
+            fetch(webRequest("$TEST_ENDPOINT/redirect/100"))
+        }
+        assertThat(thrown, equalTo(WebRequestError(WebRequestError.ERROR_REDIRECT_LOOP, WebRequestError.ERROR_CATEGORY_NETWORK)))
     }
 
     @Test
     fun testAuth() {
         // We don't support authentication yet, but want to make sure it doesn't do anything
         // silly like try to prompt the user.
-        val response = fetch(WebRequest("$TEST_ENDPOINT/basic-auth/foo/bar"))
+        val response = fetch(webRequest("$TEST_ENDPOINT/basic-auth/foo/bar"))
         assertThat("Status code should match", response.statusCode, equalTo(401))
     }
 
@@ -188,7 +214,7 @@ class WebExecutorTest {
         }
 
         try {
-            fetch(WebRequest(uri))
+            fetch(webRequest(uri))
             throw IllegalStateException("fetch() should have thrown")
         } catch (e: WebRequestError) {
             assertThat("Category should match", e.category, equalTo(WebRequestError.ERROR_CATEGORY_SECURITY))
@@ -200,61 +226,73 @@ class WebExecutorTest {
 
     @Test
     fun testSecure() {
-        val response = fetch(WebRequest("https://example.com"))
+        val response = fetch(webRequest("https://example.com"))
         assertThat("Status should match", response.statusCode, equalTo(200))
         assertThat("isSecure should match", response.isSecure, equalTo(true))
 
-        val expectedSubject = if (env.isAutomation)
+        val expectedSubject = if (env.isAutomation) {
             "CN=example.com"
-        else
+        } else {
             "CN=www.example.org,OU=Technology,O=Internet Corporation for Assigned Names and Numbers,L=Los Angeles,ST=California,C=US"
+        }
 
-        val expectedIssuer = if (env.isAutomation)
+        val expectedIssuer = if (env.isAutomation) {
             "OU=Profile Guided Optimization,O=Mozilla Testing,CN=Temporary Certificate Authority"
-        else
+        } else {
             "CN=DigiCert SHA2 Secure Server CA,O=DigiCert Inc,C=US"
+        }
 
-        assertThat("Subject should match",
-                response.certificate?.subjectX500Principal?.name,
-                equalTo(expectedSubject))
-        assertThat("Issuer should match",
-                response.certificate?.issuerX500Principal?.name,
-                equalTo(expectedIssuer))
+        assertThat(
+            "Subject should match",
+            response.certificate?.subjectX500Principal?.name,
+            equalTo(expectedSubject),
+        )
+        assertThat(
+            "Issuer should match",
+            response.certificate?.issuerX500Principal?.name,
+            equalTo(expectedIssuer),
+        )
     }
 
     @Test
     fun testCookies() {
         val uptimeMillis = SystemClock.uptimeMillis()
-        val response = fetch(WebRequest("$TEST_ENDPOINT/cookies/set/uptimeMillis/$uptimeMillis"))
+        val response = fetch(webRequest("$TEST_ENDPOINT/cookies/set/uptimeMillis/$uptimeMillis"))
 
         // We get redirected to /cookies which returns the cookies that were sent in the request
         assertThat("URI should match", response.uri, equalTo("$TEST_ENDPOINT/cookies"))
         assertThat("Status code should match", response.statusCode, equalTo(200))
 
         val body = response.getJSONBody()
-        assertThat("Body should match",
-                body.getJSONObject("cookies").getString("uptimeMillis"),
-                equalTo(uptimeMillis.toString()))
+        assertThat(
+            "Body should match",
+            body.getJSONObject("cookies").getString("uptimeMillis"),
+            equalTo(uptimeMillis.toString()),
+        )
 
-        val anotherBody = fetch(WebRequest("$TEST_ENDPOINT/cookies")).getJSONBody()
-        assertThat("Body should match",
-                anotherBody.getJSONObject("cookies").getString("uptimeMillis"),
-                equalTo(uptimeMillis.toString()))
+        val anotherBody = fetch(webRequest("$TEST_ENDPOINT/cookies")).getJSONBody()
+        assertThat(
+            "Body should match",
+            anotherBody.getJSONObject("cookies").getString("uptimeMillis"),
+            equalTo(uptimeMillis.toString()),
+        )
     }
 
     @Test
     fun testAnonymousSendCookies() {
         val uptimeMillis = SystemClock.uptimeMillis()
-        val response = fetch(WebRequest("$TEST_ENDPOINT/cookies/set/uptimeMillis/$uptimeMillis"), GeckoWebExecutor.FETCH_FLAGS_ANONYMOUS)
+        val response = fetch(webRequest("$TEST_ENDPOINT/cookies/set/uptimeMillis/$uptimeMillis"), GeckoWebExecutor.FETCH_FLAGS_ANONYMOUS)
 
         // We get redirected to /cookies which returns the cookies that were sent in the request
         assertThat("URI should match", response.uri, equalTo("$TEST_ENDPOINT/cookies"))
         assertThat("Status code should match", response.statusCode, equalTo(200))
 
         val body = response.getJSONBody()
-        assertThat("Cookies should not be set for the test server",
-                body.getJSONObject("cookies").length(),
-                equalTo(0))
+        assertThat(
+            "Cookies should not be set for the test server",
+            body.getJSONObject("cookies").length(),
+            equalTo(0),
+        )
     }
 
     @Test
@@ -262,8 +300,10 @@ class WebExecutorTest {
         // Ensure a cookie is set for the test server
         testCookies()
 
-        val response = fetch(WebRequest("$TEST_ENDPOINT/cookies"),
-                GeckoWebExecutor.FETCH_FLAGS_ANONYMOUS)
+        val response = fetch(
+            webRequest("$TEST_ENDPOINT/cookies"),
+            GeckoWebExecutor.FETCH_FLAGS_ANONYMOUS,
+        )
 
         assertThat("Status code should match", response.statusCode, equalTo(200))
         val cookies = response.getJSONBody().getJSONObject("cookies")
@@ -272,27 +312,44 @@ class WebExecutorTest {
 
     @Test
     fun testPrivateCookies() {
+        val clearData = GeckoResult<Void>()
+        ThreadUtils.runOnUiThread {
+            clearData.completeFrom(
+                RuntimeCreator.getRuntime()
+                    .storageController
+                    .clearData(StorageController.ClearFlags.ALL),
+            )
+        }
+
+        clearData.pollDefault()
+
         val uptimeMillis = SystemClock.uptimeMillis()
-        val response = fetch(WebRequest("$TEST_ENDPOINT/cookies/set/uptimeMillis/$uptimeMillis"), GeckoWebExecutor.FETCH_FLAGS_PRIVATE)
+        val response = fetch(webRequest("$TEST_ENDPOINT/cookies/set/uptimeMillis/$uptimeMillis"), GeckoWebExecutor.FETCH_FLAGS_PRIVATE)
 
         // We get redirected to /cookies which returns the cookies that were sent in the request
         assertThat("URI should match", response.uri, equalTo("$TEST_ENDPOINT/cookies"))
         assertThat("Status code should match", response.statusCode, equalTo(200))
 
         val body = response.getJSONBody()
-        assertThat("Cookies should be set for the test server",
-                body.getJSONObject("cookies").getString("uptimeMillis"),
-                equalTo(uptimeMillis.toString()))
+        assertThat(
+            "Cookies should be set for the test server",
+            body.getJSONObject("cookies").getString("uptimeMillis"),
+            equalTo(uptimeMillis.toString()),
+        )
 
-        val anotherBody = fetch(WebRequest("$TEST_ENDPOINT/cookies"), GeckoWebExecutor.FETCH_FLAGS_PRIVATE).getJSONBody()
-        assertThat("Body should match",
-                anotherBody.getJSONObject("cookies").getString("uptimeMillis"),
-                equalTo(uptimeMillis.toString()))
+        val anotherBody = fetch(webRequest("$TEST_ENDPOINT/cookies"), GeckoWebExecutor.FETCH_FLAGS_PRIVATE).getJSONBody()
+        assertThat(
+            "Body should match",
+            anotherBody.getJSONObject("cookies").getString("uptimeMillis"),
+            equalTo(uptimeMillis.toString()),
+        )
 
-        val yetAnotherBody = fetch(WebRequest("$TEST_ENDPOINT/cookies")).getJSONBody()
-        assertThat("Cookies set in private session are not supposed to be seen in normal download",
-                yetAnotherBody.getJSONObject("cookies").length(),
-                equalTo(0))
+        val yetAnotherBody = fetch(webRequest("$TEST_ENDPOINT/cookies")).getJSONBody()
+        assertThat(
+            "Cookies set in private session are not supposed to be seen in normal download",
+            yetAnotherBody.getJSONObject("cookies").length(),
+            equalTo(0),
+        )
     }
 
     @Test
@@ -302,36 +359,55 @@ class WebExecutorTest {
         executor.speculativeConnect("http://localhost")
 
         // This is just a fence to ensure the above actually ran.
-        fetch(WebRequest("$TEST_ENDPOINT/cookies"))
+        fetch(webRequest("$TEST_ENDPOINT/cookies"))
     }
 
     @Test
     fun testResolveV4() {
         val addresses = executor.resolve("localhost").pollDefault()!!
-        assertThat("Addresses should not be null",
-                addresses, notNullValue())
-        assertThat("First address should be loopback",
-                addresses.first().isLoopbackAddress, equalTo(true))
-        assertThat("First address size should be 4",
-                addresses.first().address.size, equalTo(4))
+        assertThat(
+            "Addresses should not be null",
+            addresses,
+            notNullValue(),
+        )
+        assertThat(
+            "First address should be loopback",
+            addresses.first().isLoopbackAddress,
+            equalTo(true),
+        )
+        assertThat(
+            "First address size should be 4",
+            addresses.first().address.size,
+            equalTo(4),
+        )
     }
 
     @Test
-    @SdkSuppress(minSdkVersion = Build.VERSION_CODES.LOLLIPOP)
     fun testResolveV6() {
         val addresses = executor.resolve("ip6-localhost").pollDefault()!!
-        assertThat("Addresses should not be null",
-                addresses, notNullValue())
-        assertThat("First address should be loopback",
-                addresses.first().isLoopbackAddress, equalTo(true))
-        assertThat("First address size should be 16",
-                addresses.first().address.size, equalTo(16))
+        assertThat(
+            "Addresses should not be null",
+            addresses,
+            notNullValue(),
+        )
+        assertThat(
+            "First address should be loopback",
+            addresses.first().isLoopbackAddress,
+            equalTo(true),
+        )
+        assertThat(
+            "First address size should be 16",
+            addresses.first().address.size,
+            equalTo(16),
+        )
     }
 
     @Test
     fun testFetchUnknownHost() {
-        thrown.expect(equalTo(WebRequestError(WebRequestError.ERROR_UNKNOWN_HOST, WebRequestError.ERROR_CATEGORY_URI)))
-        fetch(WebRequest("https://this.should.not.resolve"))
+        val thrown = assertThrows(WebRequestError::class.java) {
+            fetch(webRequest("https://this.should.not.resolve"))
+        }
+        assertThat(thrown, equalTo(WebRequestError(WebRequestError.ERROR_UNKNOWN_HOST, WebRequestError.ERROR_CATEGORY_URI)))
     }
 
     @Test(expected = UnknownHostException::class)
@@ -342,7 +418,7 @@ class WebExecutorTest {
     @Test
     fun testFetchStream() {
         val expectedCount = 1 * 1024 * 1024 // 1MB
-        val response = executor.fetch(WebRequest("$TEST_ENDPOINT/bytes/$expectedCount")).pollDefault()!!
+        val response = executor.fetch(webRequest("$TEST_ENDPOINT/bytes/$expectedCount")).pollDefault()!!
 
         assertThat("Status code should match", response.statusCode, equalTo(200))
         assertThat("Content-Length should match", response.headers["Content-Length"]!!.toInt(), equalTo(expectedCount))
@@ -354,19 +430,23 @@ class WebExecutorTest {
         assertThat("Byte counts should match", bytes.size, equalTo(expectedCount))
 
         val digest = MessageDigest.getInstance("SHA-256").digest(bytes)
-        assertThat("Hashes should match", response.headers["X-SHA-256"],
-                equalTo(String.format("%064x", BigInteger(1, digest))))
+        assertThat(
+            "Hashes should match",
+            response.headers["X-SHA-256"],
+            equalTo(String.format("%064x", BigInteger(1, digest))),
+        )
     }
 
     @Test(expected = IOException::class)
     fun testFetchStreamError() {
-
         val expectedCount = 1 * 1024 * 1024 // 1MB
-        val response = executor.fetch(WebRequest("$TEST_ENDPOINT/bytes/$expectedCount"),
-                GeckoWebExecutor.FETCH_FLAGS_STREAM_FAILURE_TEST).pollDefault()!!
+        val response = executor.fetch(
+            webRequest("$TEST_ENDPOINT/bytes/$expectedCount"),
+            GeckoWebExecutor.FETCH_FLAGS_STREAM_FAILURE_TEST,
+        ).pollDefault()!!
 
         assertThat("Status code should match", response.statusCode, equalTo(200))
-        assertThat("Content-Length should match",response.headers["Content-Length"]!!.toInt(), equalTo(expectedCount))
+        assertThat("Content-Length should match", response.headers["Content-Length"]!!.toInt(), equalTo(expectedCount))
 
         val stream = response.body!!
         val bytes = ByteArray(1)
@@ -375,7 +455,7 @@ class WebExecutorTest {
 
     @Test(expected = IOException::class)
     fun readClosedStream() {
-        val response = executor.fetch(WebRequest("$TEST_ENDPOINT/bytes/1024")).pollDefault()!!
+        val response = executor.fetch(webRequest("$TEST_ENDPOINT/bytes/1024")).pollDefault()!!
 
         assertThat("Status code should match", response.statusCode, equalTo(200))
 
@@ -387,7 +467,7 @@ class WebExecutorTest {
     @Test(expected = IOException::class)
     fun readTimeout() {
         val expectedCount = 10
-        val response = executor.fetch(WebRequest("$TEST_ENDPOINT/trickle/${expectedCount}")).pollDefault()!!
+        val response = executor.fetch(webRequest("$TEST_ENDPOINT/trickle/$expectedCount")).pollDefault()!!
 
         assertThat("Status code should match", response.statusCode, equalTo(200))
         assertThat("Content-Length should match", response.headers["Content-Length"]!!.toInt(), equalTo(expectedCount))
@@ -402,20 +482,20 @@ class WebExecutorTest {
     @Test
     fun testFetchStreamCancel() {
         val expectedCount = 1 * 1024 * 1024 // 1MB
-        val response = executor.fetch(WebRequest("$TEST_ENDPOINT/bytes/$expectedCount")).pollDefault()!!
+        val response = executor.fetch(webRequest("$TEST_ENDPOINT/bytes/$expectedCount")).pollDefault()!!
 
         assertThat("Status code should match", response.statusCode, equalTo(200))
         assertThat("Content-Length should match", response.headers["Content-Length"]!!.toInt(), equalTo(expectedCount))
 
-        val stream = response.body!!;
+        val stream = response.body!!
 
         assertThat("Stream should have 0 bytes available", stream.available(), equalTo(0))
 
         // Wait a second. Not perfect, but should be enough time for at least one buffer
         // to be appended if things are not going as they should.
-        SystemClock.sleep(1000);
+        SystemClock.sleep(1000)
 
-        assertThat("Stream should still have 0 bytes available", stream.available(), equalTo(0));
+        assertThat("Stream should still have 0 bytes available", stream.available(), equalTo(0))
 
         stream.close()
     }
@@ -432,34 +512,38 @@ class WebExecutorTest {
             "1234567890" to "1234567890",
             "12345678901" to "1234567890",
             "file://test" to "file://tes",
-            "moz-extension://what" to "moz-extens"
+            "moz-extension://what" to "moz-extens",
         )
 
         for ((uri, truncated) in illegal) {
             try {
-                fetch(WebRequest(uri))
+                fetch(webRequest(uri))
                 throw IllegalStateException("fetch() should have thrown")
             } catch (e: IllegalArgumentException) {
-                assertThat("Message should match",
-                        e.message,
-                        equalTo("Unsupported URI scheme: $truncated"))
+                assertThat(
+                    "Message should match",
+                    e.message,
+                    equalTo("Unsupported URI scheme: $truncated"),
+                )
             }
         }
 
         val legal = listOf(
             "http://$TEST_ENDPOINT\n",
             "http://$TEST_ENDPOINT/🥲",
-            "http://$TEST_ENDPOINT/abc"
+            "http://$TEST_ENDPOINT/abc",
         )
 
         for (uri in legal) {
             try {
-                fetch(WebRequest(uri))
+                fetch(webRequest(uri))
                 throw IllegalStateException("fetch() should have thrown")
             } catch (e: WebRequestError) {
-                assertThat("Request should pass initial validation.",
-                        true,
-                        equalTo(true))
+                assertThat(
+                    "Request should pass initial validation.",
+                    true,
+                    equalTo(true),
+                )
             }
         }
     }
